import datetime
from re import A
import matplotlib.pyplot as plt

import iexcode.instruments.cfg as iex
from iexcode.instruments.ARPES import ARPES_init
from iexcode.instrumetns.kappa import kappa_init

from iexcode.instruments.diagnostics import _diagnostics_dict, diagnostics_all_out, diagnostics_all_in,diode_c,diode_d
from iexcode.instruments.current_amplifiers import *
from iexcode.instruments.xrays import *
from iexcode.instruments.VLS_PGM import *
from iexcode.instruments.Logfile import log_name_set,log_print
from iexcode.instruments.shutters import branch_shutter_open, branch_shutter_status
from iexcode.instruments.utilities import *
from iexcode.macros.commissioning import *
from iexcode.macros.quick_plot import *

def StartOfTheWeek(GRT,branch,wait=False,**kwargs):
    """
    This should be run every at the start of every week; 50 min total.
    Switch to C-branch,.
    If wait=True, wait for next day a 8:05.
    
    kwargs defaults:
        mode =  polarization
        run = None: used today's date to set run appropriately 
        repeat = False: sets everything: switches branch, sets BL_Mode to Staff, resets scan/CAs/mono
        scanType = ['slit1','wire','flux','monoVslit']
        sound = False; no sounds
              = True => plays sound after Slit1A scans and when script is complete
    }
    ID steering: (slit1A)
        steering out => move beam more positive (10 urad ~ 0.25 mm)
        steering up  => move beam more positive (10 urad ~ 0.25 mm)
        
    M1 steering: (monoVslit-2V and 2H)
        Roll:  Rz more positive => 2V more positive (0.25 => .25 mm)
        Pitch: Ry more positive => 2H more positive (0.03 => .5 mm
    """
    kwargs.setdefault('mode','RCP')
    kwargs.setdefault('repeat',False)
    kwargs.setdefault('quick',True)    
    kwargs.setdefault('scanType',['slit1','wire','flux','monoVslit'])
    kwargs.setdefault('sound',True)
    kwargs.setdefault('extended_range',False)

    branch=branch.lower()
    if iex.BL == None:
        if branch == 'c':
            BL_init=ARPES_init
        else:
            BL_init=kappa_init

    if kwargs['repeat']==False: #reset folders/detectors/...
        BL_init('staff')
    else:
        BL_init('staff',reset=False,set_folders=False)
        switch_branch(branch)
        FileName='StartOfTheWeek_log.txt'
        log_name_set(FileName)
            
    for scan in kwargs['scanType']: 
        if scan not in ['slit1','wire','flux','monoVslit']:
            print(scan+" is not a valid scan scanType=['slit1','wire','flux','monoVslit']")
            return
       
    ### checking diode settings:
    if branch == 'c': 
        ca15 = Keithley('b',15)
        ca15.autoscale('Off')
        diode_c('In')
    else:
        foo=input('Do you have a diode in direct beam (y or n)? >')
        if foo.lower() == 'y' or foo.lower() == 'yes':
            print('Resuming...')
    
    ### Stuff that doesn't need beam & does not need to be run if repeating SoTW:
    print("\n\n================== Sets Beamline & Scans:")
    grating(GRT)

    ### checking branch shutter:
    shutter_open = branch_shutter_status()
    if shutter_open == False:
        foo=input_d('Shutter '+branch.upper()+' is closed, do you want to open it (y or n)? >')
        if foo.lower() == 'y' or foo.lower() ==  'yes':
            branch_shutter_open()
        else:
            print('Aborting...')
            return

    
    ### checking QP ratio:
    QP=caget('ID29:QuasiRatio.RVAL')
    if QP != 100:
        foo=input('QP on!!! Continue (y or n)? >')
        if foo.lower() == 'y' or foo.lower() == 'yes':
            print('Resuming...')

    ### Wait for next 8AM:
    if wait:
        t = datetime.today()
        if 0 <= t.hour <= 8:
            wait_for_it(0,8,5)
        else:    
            wait_for_it(1,8,5)

    ### checking ID:
    polarization(kwargs['mode'])
            
    ### Ready to start:
    FirstScan=iex.BL.mda.filenum()

    if not kwargs['repeat']:    
        log_print("\n\n================== Start Of The Week @ "+today('slash')+':\n',FileName=FileName)
        
        
       
    def interactive_fct(mirror=False): 
        options='0,1,2,3'
        if not mirror:
            print('''\nDo you want to:
            \t (0) write comment
            \t (1) continue
            \t (2) repeat scan
            \t (3) adjust xrange for fit
            ''')
        else:
            print('''\nDo you want to:
            \t (0) write comment
            \t (1) continue
            \t (2) repeat scan
            \t (3) adjust xrange for fit
            \t (4) move M1 roll
            ''')
            options=options+',4'
        flag=input('\nType ('+options+') >')
        if flag not in options.split(","):
            print('\ninvalid input; please try again...\n')
            flag=input('\nType ('+options+') >')
        if flag == '0': 
            cmt=input('\nType comment >')
            print('\n================================')
            log_print(cmt)
            print('================================\n')
            flag=input('\nType ('+options+') >')
        if flag == '1': print('\nresuming script...\n'); flag=1
        if flag == '2': print('\nrepeating scan...\n'); flag=2
        if flag == '3': flag=3
        if mirror:
            if flag == '4': flag=4
        
        return flag



        # if foo.lower() == 'y' or foo.lower() ==  'yes': 
        #     print('\nType any comments here - you have 30s! >')
        #     i, o, e = select.select( [sys.stdin], [], [], 5 )
        #     if (i):
        #         print ('\n==============\n'+sys.stdin.readline().strip()+'\n==============\n')
        #     else:
        #         print('\nTimeout...');
        #     print('\nRepeating scan...\n');flag=2
            
        # else: 
        #     print('\nResuming script...\n'); flag=3
        # return flag
    
    

    def iex_steering(h_position,v_position):
        print('\nBeam position:')
        print('   - horizontal: '+str(h_position))
        print('   - vertical:   '+str(v_position))
        h_steering,v_steering=beamsteering_1A(h_position,v_position)
        print('\nBeam steering:')
        print('   - horizontal: '+h_steering)
        print('   - vertical:   '+v_steering)
        print('\n')
        return h_steering,v_steering
    
    DetDict={'c':(9,7,8,15),'d':(9,7,8,14)}
    detCA4,detH,detV,detDiode=DetDict[branch]
    list_position=[GRT,0,0,0,0]

    ###### Scan front-end slit center:
    if 'slit1' in kwargs['scanType']:
        print("\n\n================== Slit 1A scans:")
        mvID(2000)
        flag=2
        while flag==2:
            slits_scan_center('slit1A','H',-3,3,0.1,size=(0.25,inf),comment='Slit center - 1H')
            slits_scan_center('slit1A','V',-3,3,0.1,size=(inf,0.25),comment='Slit center - 1V')
            slit1A_set(4.5,4.5,0,0)
            m=iex.BL.mda.last_mda()
            print('\nsteering out => move beam more positive (10 urad ~ 0.25 mm)')
            print('steering up  => move beam more positive (10 urad ~ 0.25 mm)\n')
            try:
                h_position=fit_mda(m-1,detCA4,1,'gauss',xrange=[-1,1]);plt.show()
                v_position=fit_mda(m  ,detCA4,1,'gauss',xrange=[-1,1]);plt.show()
                iex_steering(h_position,v_position)   
            except:
                print('\nUnable to fit position; try to adjust xrange.\n')
                # print('h_position=fit_mda('+str(m-1)+','+str(detCA4)+',1,"gauss",xrange=[-1,1])')
                # print('v_position=fit_mda('+str(m)+','+str(detCA4)+',1,"gauss",xrange=[-1,1])')
                # print('\nThen use:')
                # print('\tbeamsteering_1A(h_position,v_position)')
            if kwargs['sound']:playsound()
            flag=interactive_fct()
            while flag == 3:
                try:
                    fitH=input('do you want to fit mda #'+str(m-1)+' ie horizontal (y or n)? >')
                    if fitH.lower() == 'y' or fitH.lower() ==  'yes': 
                        x1=input('x1 >')
                        x2=input('x2 >')
                        h_position=fit_mda(m-1,detCA4,1,'gauss',xrange=[float(x1),float(x2)]);plt.show()
                    fitV=input('do you want to fit mda #'+str(m)+' ie vertical (y or n)? >')
                    if fitV.lower() == 'y' or fitV.lower() ==  'yes': 
                        x1=input('x1 >')
                        x2=input('x2 >')
                        v_position=fit_mda(m  ,detCA4,1,'gauss',xrange=[float(x1),float(x2)]);plt.show()
                    iex_steering(h_position,v_position)
                    foo=input('\nare you happy with the fit (y or n)? >')
                    if foo.lower() == 'y' or foo.lower() ==  'yes': 
                        flag=interactive_fct()
                except:
                    print('\nUnable to fit position; try to tweak xrange again\n') 
                    flag=interactive_fct()

    ###### Wire scans: 
    if 'wire' in kwargs['scanType']:
        mvID(2000)
        slit1A_set(4.5,4.5,0,0)
        scan_wire('H',comment='Wire scan - H',all_diag=False)     # by default puts all meshes in + DiodeC
        scan_wire('V',comment='Wire scan - V',all_diag=False)
        m=iex.BL.mda.last_mda()
        try:
            plot_mda(m-1,detH,m,detV,title='wire-H (blue) & wire-V (orange)');plt.show()
        except:
            print('Unable to plot. Try:')
            print("plot_mda("+str(m-1)+","+str(detH)+","+str(m)+","+str(detV)+",title='wire-H (blue) & wire-V (orange)');plt.show()")
        if kwargs['sound']:playsound()
       

    ###### Mono/slit scans: 
    if 'monoVslit' in kwargs['scanType']: 
        c=mono_grating_density_get()/1200     
        if c==2: 
            slit(300)
        else: 
            slit(200)
        print("\n\n================== Mono/slit scans:")     
        if 'quick' in kwargs:
            energy(500)
            mvmono(505)
            print('\n---------- Scanning slit 2B:\n') 
            flag=2
            while flag==2:
                slits_scan_center('slit2B','V',-8,8,0.25,size=(inf,0.25*c));sleep(1)
                slits_scan_center('slit2B','H',-6,6,0.25,size=(0.25,inf));sleep(1)
                m=iex.BL.mda.last_mda()
                try:
                    V2=fit_mda(m-1,detDiode,1*c,'gauss');plt.show()
                    H2=fit_mda(m  ,detDiode,1,'gauss');plt.show()
                    list_position[1]=V2
                    list_position[2]=H2
                    #print(f"V2 = {round(V2,3)}")
                    #print(f"H2 = {round(H2,3)}")
                      
                    print('\nBeam position:')
                    print(f"   - vertical:   V2 = {round(V2,3)}")
                    print(f"   - horizontal: H2 = {round(H2,3)}")
                    v_steering=beamsteering_2B(V2)
                    m1_roll=beamsteering_2B(V2)[1]
                    print('\nCurrent position:')
                    current_roll=FMB_mirror_get(1)[5]
                    new_roll=current_roll+m1_roll
                    print('\nREMEMBER: 2V slit should always be centered at zero by steering M1 roll!')
                    print("   - vertical: "+str(current_roll)+str(v_steering[1])+" => Move_M1('RZ',"+str(new_roll)+")")
                    print('   - horizontal: reset slit 2BH center to ',str(H2))     
                    
                except:
                    print('Unable to fit position; try to adjust xrange.')
                    # print('c='+str(c))
                    # print('V2=fit_mda('+str(m-1)+','+str(detDiode)+',1*c+,"gauss")')
                    # print('H2=fit_mda('+str(m)+','+str(detDiode)+',1,"gauss")')
                    # print('\nThen use:')
                    # print('\tbeamsteering_2B(V2)')
                    # list_position.append(np.nan)
                    # list_position.append(np.nan)
                    flag=interactive_fct(mirror=True)   

                if kwargs['sound']:playsound()
                flag=interactive_fct(mirror=True)            
                while flag == 3:
                    fitV2=input('do you want to FIT mda #'+str(m-1)+' ie slit2-V (y or n)? >')
                    if fitV2.lower() == 'y' or fitV2.lower() ==  'yes': 
                        x1=input('x1 >')
                        x2=input('x2 >')
                        V2=fit_mda(m-1,detDiode,1,'gauss',xrange=[float(x1),float(x2)]);plt.show()
                        foo=input('do you want to replace slit2-V value in list_position (y or n)? >')
                        if foo.lower() == 'y' or foo.lower() ==  'yes':
                            list_position[1]=V2
                    fitH2=input('do you want to FIT mda #'+str(m)+' ie slit2-H (y or n)? >')
                    if fitH2.lower() == 'y' or fitH2.lower() ==  'yes': 
                        x1=input('x1 >')
                        x2=input('x2 >')
                        H2=fit_mda(m  ,detDiode,1,'gauss',xrange=[float(x1),float(x2)]);plt.show()    
                        foo=input('do you want to REPLACE slit2-H value in list_position (y or n)? >')
                        if foo.lower() == 'y' or foo.lower() ==  'yes':
                            list_position[1]=H2
                    foo=input('\nare you happy with the fit (y or n)? >')
                    if foo.lower() == 'y' or foo.lower() ==  'yes': 
                        flag=interactive_fct(mirror=True) 
                if flag == 4:
                    x=input('New M1 roll value >')
                    x=float(x)
                    try:
                        FMB_mirror_move(1,'RZ',x)
                        sleep(2);
                        FMB_mirror_get(1)
                        print('\nrepeating scan...\n')
                        flag=2
                    except:
                        print('\ninvalid input; unable to move M1...\n')
                        flag=interactive_fct(mirror=True) 
            print('\n---------- Scanning slit 1A:\n') 
            flag=2
            while flag==2:
                slits_scan_center('slit1A','V',-4, 4, 0.1,size=(inf,0.25))
                slits_scan_center('slit1A','H',-3, 3, 0.1,size=(0.25,inf))
                apertures_set()
                m=iex.BL.mda.last_mda()
                try:
                    V1=fit_mda(m-1,detDiode,1,'gauss');plt.show()
                    H1=fit_mda(m  ,detDiode,1,'gauss');plt.show()
                    list_position[3]=V1
                    list_position[4]=H1
                    print('\nBeam position:')
                    print(f"   - vertical:   V1 = {round(V1,3)}")
                    print(f"   - horizontal: H1 = {round(H1,3)}")
                except:
                    print('\nUnable to fit position; try to adjust xrange.')
                    # print('V1=fit_mda('+str(m-1)+','+str(detDiode)+',1,"gauss")')
                    # print('H1=fit_mda('+str(m)+','+str(detDiode)+',1,"gauss")')
                    # list_position.append(np.nan)
                    # list_position.append(np.nan)
                if kwargs['sound']:playsound()
                flag=interactive_fct()             #interactive_fct('slit 1A')
                while flag == 3:
                    fitV1=input('do you want to fit mda #'+str(m-1)+' ie slit1-V (y or n)? >')
                    if fitV1.lower() == 'y' or fitV1.lower() ==  'yes': 
                        x1=input('x1 >')
                        x2=input('x2 >')
                        V1=fit_mda(m-1,detDiode,1,'gauss',xrange=[float(x1),float(x2)]);plt.show()
                        foo=input('do you want to replace slit1-V value in list_position (y or n)? >')
                        if foo.lower() == 'y' or foo.lower() ==  'yes':
                            list_position[3]=V1
                    fitH1=input('do you want to fit mda #'+str(m)+' ie slit1-H (y or n)? >')
                    if fitH1.lower() == 'y' or fitH1.lower() ==  'yes': 
                        x1=input('x1 >')
                        x2=input('x2 >')
                        H1=fit_mda(m  ,detDiode,1,'gauss',xrange=[float(x1),float(x2)]);plt.show()    
                        foo=input('do you want to replace slit1-H value in list_position (y or n)? >')
                        if foo.lower() == 'y' or foo.lower() ==  'yes':
                            list_position[4]=H1
                    foo=input('\nare you happy with the fit (y or n)? >')
                    if foo.lower() == 'y' or foo.lower() ==  'yes':
                        flag=interactive_fct()
            print('\nBeam center fit @ ['+GRT+',2V,2H,1V,1H]:',list_position)
            foo=input('do you want to update the slit dictionary (y or n)? >')
            if foo.lower() == 'y' or foo.lower() ==  'yes':
                slit_list=input('do you want to use '+str(list_position)+' (y or n)? >')
                if foo.lower() == 'y' or foo.lower() ==  'yes':
                    update_slit_dict(list_position)
                else:
                    update_slit_dict()
        else:    # longer full slit scans
            mvID(500)
            flag=2
            while flag==2:
                mvmono(500)
                apertures_set()
                n=2 if 'extended_range' in kwargs  else 1
                scan_mono_vs_slit('2V',[0.25*c,-2*c*n,2*c*n,0.5],[475,515,2],comment='Mono/Slit - 2V')    #10/20 min for MEG/HEG
                scan_mono_vs_slit('2H',[0.25,-2*n,2*n,0.5],[475,515,2],comment='Mono/Slit - 2H')    #10min
                scan_mono_vs_slit('1V',[0.5,-0.75*n,0.75*n,0.25*c],[475,515,2],comment='Mono/Slit - 1V')    #10min
                scan_mono_vs_slit('1H',[0.5,-0.75*n,0.75*n,0.25],[475,515,2],comment='Mono/Slit - 1H')    #10min
                #interactive
                flag=interactive_fct()
                    
    ###### Check flux: 
    if 'flux' in kwargs['scanType']:
        diode_c('In')
        print("\n\n================== Check Flux:")
        flag=2
        while flag==2:
            check_flux(stay='y')
            slit(50)
            energy(500)
            scanmono(475,525,1,comment='Mono Scan @ 500eV')
            m=iex.BL.mda.last_mda()
            try:
                plot_mda(m,detDiode,Flux=3);plt.show()
            except:
                print('Unable to plot. Try:') 
                print("plot_mda("+str(m)+","+str(detDiode)+",flux=3);plt.show()")  
            flag=interactive_fct()

    diagnostics_all_out()

    print("\n\n================== All done:")
    print("\nDon't forget to put back the user folder !!!!")
    

    ###### The end:
    if list_position:
        print('\nBeam center fit @ ['+GRT+',2V,2H,1V,1H]:',list_position)
    if kwargs['sound']:
        playsound()
        
    return list_position
    



def update_slit_dict(mylist=None,**kwargs):
    """
    Interactive function to update the slit position dictionary (Dict_Slit.txt)
    mylist=['GRT',2V,2H,1V,1H]
    **kwargs
        readOnly == False, if true just pring the current slit dictionary
    """
    filepath = "/home/beams22/29IDUSER/Documents/User_Macros/Macros_29id/IEX_Dictionaries/"
    filename ='Dict_Slit.txt'
    
    kwargs.setdefault("readOnly",False)

    # print('\n\nWARNING: 2V slit should always be centered at zero ie steer M1 center beam on the grating')
    # print('Increasing the roll moves the beam more positive: +0.05 pitch => ~ +0.04 slit position; Use:\n')
    # print("Get_Mirror(1);Move_M1('RZ',x)")
    # print("energy(500);mono(505);slit(200)")
    # print("Scan_NarrowSlit_Go(which='2V',slit_parameters=[0.25, -2, 2, 0.25]) #for MEG; slit_parameters=[0.5, -2, 2, 0.25] for HEG")
    # print("Scan_NarrowSlit_Go(which='2V',slit_parameters=[0.50, -2, 2, 0.25]) #for HEG")
    # print('Once steering is completed, check 2H position:')
    # print("Scan_NarrowSlit_Go(which='2H',slit_parameters=[0.25, -6, 6, 0.25]) ")
    

    try:
        slit_position=read_dict(filename)
        print('\nCurrent dictionary:\n')
        print(slit_position)

    except KeyError:
        print("Unable to read dictionary")
        return
    
    if kwargs['readOnly']== True:
        return slit_position
        
    else:
        if mylist:
            if len(mylist) == 5:
                grt,s2v,s2h,s1v,s1h=mylist
            else:
                print('\nInvalid list. No changes made in file.')
                return              

        else:
            grt=input('\nWhich grating do you want to update (HEG or MEG) >')
            s2v=input('New Slit 2B-V center >')
            s2h=input('New Slit 2B-H center >')
            s1v=input('New Slit 1A-V center >')
            s1h=input('New Slit 1A-H center >')

            if grt == '' or s1h == '' or s1v == '' or s2h == '' or s2v == '':
                print('\nMissing input. No changes made in file.')
                return

            if grt[0] == '"' or grt[0]=="'": grt = grt[1:4]
            
        #    new_slit_position={grt.upper():{'S1H':float(s1h),'S1V':float(s1v),'S2H':float(s2h),'S2V':float(s2v)}}   # change order to match scan/fit order
        if grt.upper() == 'MEG' or grt.upper() == 'HEG':
            new_slit_position={grt.upper():{'S2V':float(s2v),'S2H':float(s2h),'S1V':float(s1v),'S1H':float(s1h)}}
        else:
            print('\nInvalid grating name. No changes made in file.')
            return            


        slit_position.update(new_slit_position)


        with open(join(filepath, filename), "a+") as f:
            f.write('\n======= '+today()+': \n')
            f.write(str(slit_position))
            f.write('\n')
            print('\nWriting dictionary to:',join(filepath, filename))
        sleep(2)
        apertures_set()
        return slit_position


    
def update_slit_dict(**kwargs):
    """
    Interactive function to update the slit position dictionary (Dict_Slit.txt)
    
    **kwargs
        readOnly == False, if true just pring the current slit dictionary
    """
    filepath = "/home/beams22/29IDUSER/Documents/User_Macros/Macros_29id/IEX_Dictionaries/"
    filename ='Dict_Slit.txt'
    
    kwargs.setdefault("readOnly",False)

    print('\n\nWARNING: 2V slit should always be centered at zero ie steer M1 center beam on the grating')
    print('Increasing the roll moves the beam more positive: +0.05 pitch => ~ +0.04 slit position; Use:\n')
    print("Get_Mirror(1);Move_M1('RZ',x)")
    print("energy(500);mono(505);slit(200)")
    print("Scan_NarrowSlit_Go(which='2V',slit_parameters=[0.25, -2, 2, 0.25]) #for MEG; slit_parameters=[0.5, -2, 2, 0.25] for HEG")
    print("Scan_NarrowSlit_Go(which='2V',slit_parameters=[0.50, -2, 2, 0.25]) #for HEG")
    print('Once steering is completed, check 2H position:')
    print("Scan_NarrowSlit_Go(which='2H',slit_parameters=[0.25, -6, 6, 0.25]) ")
    
    try:
        slit_position=read_dict(filename)
        print('\nCurrent dictionary:\n')
        print(slit_position)

    except KeyError:
        print("Unable to read previous dictionary")
        return
    
    if kwargs['readOnly']== True:
        return slit_position
        
    else:
        grt=input('\nWhich grating do you want to update (HEG or MEG) >')
        s2v=input('New Slit 2B-V center >')
        s2h=input('New Slit 2B-H center >')
        s1v=input('New Slit 1A-V center >')
        s1h=input('New Slit 1A-H center >')

        if grt == '' or s1h == '' or s1v == '' or s2h == '' or s2v == '':
            print('\nMissing input. No changes made in file.')
            return

        if grt[0] == '"' or grt[0]=="'": grt = grt[1:4]

    #    new_slit_position={grt.upper():{'S1H':float(s1h),'S1V':float(s1v),'S2H':float(s2h),'S2V':float(s2v)}}   # change order to match scan/fit order
        new_slit_position={grt.upper():{'S2V':float(s2v),'S2H':float(s2h),'S1V':float(s1v),'S1H':float(s1h)}}
        slit_position.update(new_slit_position)


        with open(join(filepath, filename), "a+") as f:
            f.write('\n======= '+today()+': \n')
            f.write(str(slit_position))
            f.write('\n')
            print('\nWriting dictionary to:',join(filepath, filename))

        apertures_set()
        return slit_position

###############################################################################################
######################################### Start Of The Week plotting -- old!!!! ###################################
###############################################################################################

def plot_StartOfTheWeek_DetDict(branch):
    """
    returns the detectors for a given branch
    
    """
    DetDict={'c':(9,7,8,15),'d':(9,7,8,14)}
    return DetDict[branch]

def StartOfTheWeek_plot(branch,FirstScanNum,**kwargs):
    """
    Plots the data from StartOfTheWeek
    
    branch is used to set the detectors
        detCA4,detH,detV,detDiode=plot_StartOfTheWeek_Det(branch)
        
    FirstScanNum is based on slit1A
        slit1A-H = FirstScanNum
        slit1A-V = FirstScanNum + 1
        wire1-H = FirstScanNum + 2
        wire1-V = FirstScanNum + 3
        monoVslit = FirstScanNum + 4/5/6/7
        flux = FirstScanNum + 8
        
    Slit 1A and Wire scans: determine ID steering and Slit1A position
    Scan_MonoVsSlit: determine the steering from M0/M1 
        by default the full range is plotted (pnt_first=0, pnt_last=inf)
        refine the plot via
            plot_StartOfTheWeek_mono(branch,FirstScanNum,pnt_first,pnt_last)

    kwargs: 
        filepath = None,  uses current mda filepath unless specified
              e.g. user : filepath='/net/s29data/export/data_29idc/2018_2/UserName/mda/'
              e.g. staff: filepath='/net/s29data/export/data_29idb/2018_2/mda/'
        prefix = None, uses current mda prefix unless specified 
        scanIOC = None, uses BL_ioc() unless specified
        
        scanType = ['slit1','wire','flux','monoVslit'], full set by default
        
        ref_firstScanNum to plot reference spectra
        ref_branch = branch, unless specified
        ref_filepath = filepath, unless specified
        ref_prefix = prefix, unless specified

    steering out => moves beam more positive (20 urad ~ 0.5 mm)
    steering up  => moves beam more positive (20 urad ~ 0.5 mm)
    
figure, axes = plt.subplots(nrows=2, ncols=2)
axes[0, 0].plot(x, y)
    """
    
    kwargs.setdefault('filepath',BL.mda.filepath())
    kwargs.setdefault('prefix',BL.mda.prefix())
    
    kwargs.setdefault('scanType',['slit1','wire','flux','monoVslit'])
    
    kwargs.setdefault('ref_firstScanNum',None)
    kwargs.setdefault('ref_fpath',None)
    kwargs.setdefault('ref_branch',branch)
    kwargs.setdefault('ref_filepath',BL.mda.filepath())
    kwargs.setdefault('ref_prefix',BL.mda.prefix())
    
    kwargs.setdefault('debug',False)

    
    scanNum=FirstScanNum
    ref_firstScanNum=kwargs['ref_firstScanNum']
    detCA4,detH,detV,detDiode=plot_StartOfTheWeek_DetDict(branch)
    

    if 'slit1' in kwargs['scanType']:
        if kwargs['debug']:
            print('slit1')
        plot_StartOfTheWeek_slit1A(branch,scanNum,**kwargs)
        scanNum+=2
        if kwargs['ref_firstScanNum'] is not None:
            kwargs.update({'ref_firstScanNum':ref_firstScanNum+2})
            
            
    if 'wire' in kwargs['scanType']:
        if kwargs['debug']:
            print('wire')
        plot_StartOfTheWeek_wire(branch,scanNum,**kwargs)
        scanNum+=2
        if kwargs['ref_firstScanNum'] is not None:
            ref_firstScanNum+=2
            kwargs.update({'ref_firstScanNum':ref_firstScanNum+2})
            
    if 'monoVslit' in kwargs['scanType']:
        d=IEXdata(scanNum,path=kwargs['filepath'],prefix=kwargs['prefix'],q=1)
        if d.mda[scanNum].header.all['rank']<2:  #quick
            print(scanNum, detDiode)
            V2=fit_mda(scanNum,detDiode,1,'gauss',filepath=kwargs['filepath'],prefix=kwargs['prefix'],scanIOC=kwargs['scanIOC'],title='2V - mda_'+str(scanNum))
            H2=fit_mda(scanNum+1,detDiode,1,'gauss',filepath=kwargs['filepath'],prefix=kwargs['prefix'],scanIOC=kwargs['scanIOC'],title='2H - mda_'+str(scanNum+1))
            V1=fit_mda(scanNum+2,detDiode,1,'gauss',filepath=kwargs['filepath'],prefix=kwargs['prefix'],scanIOC=kwargs['scanIOC'],title='1V - mda_'+str(scanNum+2))
            H1=fit_mda(scanNum+3,detDiode,1,'gauss',filepath=kwargs['filepath'],prefix=kwargs['prefix'],scanIOC=kwargs['scanIOC'],title='1H - mda_'+str(scanNum+3))
            plt.show()
            print('\nFit positions:')
            print(f"V2={round(V2,3)}")
            print(f"H2={round(H2,3)}")
            print(f"V1={round(V1,3)}")
            print(f"H1={round(H1,3)}")
            
        else:  #MonoVsSlit
            print('\n# To plot the Mono vs Slit data use:\n')
            print('\tplot_MonoVsSlit(\"'+branch+'\",'+str(scanNum)+','+str(detDiode)+',0,inf)'+'\t#2V')
            print('\tplot_MonoVsSlit(\"'+branch+'\",'+str(scanNum+1)+','+str(detDiode)+',0,inf)'+'\t#2H')
            print('\tplot_MonoVsSlit(\"'+branch+'\",'+str(scanNum+2)+','+str(detDiode)+',0,inf)'+'\t#1V')
            print('\tplot_MonoVsSlit(\"'+branch+'\",'+str(scanNum+3)+','+str(detDiode)+',0,inf)'+'\t#1H')
            print('\n#  (pnt_first,pnt_last0=(0,inf) => plots all')
            print('#  select specific first/last to refine.')

        print('\nREMEMBER to update slit center using: \tupdate_slit_dict()' )
        scanNum+=4
        if kwargs['ref_firstScanNum'] is not None:
            kwargs.update({'ref_firstScanNum':ref_firstScanNum+4})
        
    if 'flux' in kwargs['scanType']:
        if kwargs['debug']:
            print('flux')
        plot_StartOfTheWeek_flux(branch,scanNum,**kwargs)

def plot_StartOfTheWeek_slit1A(branch,scanNum,**kwargs):
    """
    plots the slit1A scans
    scanNum = slit1A-H scanNum
    """   
    kwargs.setdefault('filepath',BL.mda.filepath())
    kwargs.setdefault('prefix',BL.mda.prefix())
    kwargs.setdefault('plotType',['slit1','wire','flux','monoVslit'])
    kwargs.setdefault('ref_firstScanNum',None)
    kwargs.setdefault('ref_fpath',None)
    kwargs.setdefault('ref_branch',branch)
    kwargs.setdefault('ref_filepath',BL.mda.filepath())
    kwargs.setdefault('ref_prefix',BL.mda.prefix())
    kwargs.setdefault('debug',False)

    detCA4,detH,detV,detDiode=plot_StartOfTheWeek_DetDict(branch)
    ref_detCA4,ref_detH,ref_detV,ref_detDiode=plot_StartOfTheWeek_DetDict(kwargs['ref_branch']) 
    
    figure, axes = plt.subplots(nrows=1, ncols=2,figsize=(10,3))
    for i,t in enumerate(['1H center scan','1V center scan']):
        d=IEXdata(scanNum,path=kwargs['filepath'],prefix=kwargs['prefix'],q=1)
        axes[i].plot(d.mda[scanNum].det[detCA4].scale['x'], d.mda[scanNum].det[detCA4].data,marker='x',label=str(scanNum))
        scanNum+=1
        if kwargs['ref_firstScanNum'] is not None:
            ref_scanNum=kwargs['ref_firstScanNum']+i
            ref_d=IEXdata(ref_scanNum,path=kwargs['ref_filepath'],prefix=kwargs['ref_prefix'],q=1)
            axes[i].plot(ref_d.mda[ref_scanNum].det[ref_detCA4].scale['x'], ref_d.mda[ref_scanNum].det[ref_detCA4].data,marker='x',label=str(ref_scanNum))
        axes[i].grid(color='lightgray', linestyle='-', linewidth=0.5)
        axes[i].title.set_text(t)
        axes[i].legend()
    plt.show()
    print("\nsteering out => move beam more positive (10 urad ~ 0.25 mm)")
    print("steering up  => move beam more positive (10 urad ~ 0.25 mm)")
    print("\nTo fit beam position use:\n")
    print("detCA4,detH,detV,detDiode=plot_StartOfTheWeek_DetDict('"+kwargs['ref_branch']+"')")
    print("fit_mda("+str(scanNum-2)+",detCA4,1,'gauss',xrange=(-1,1))")
    print("fit_mda("+str(scanNum-1)+",detCA4,1,'gauss',xrange=(-1,1))\n")

    
def plot_StartOfTheWeek_wire(branch,scanNum,**kwargs):
    """
    plots the wire scans
    scanNum = wire-H scanNum
    """   
    kwargs.setdefault('filepath',BL.mda.filepath())
    kwargs.setdefault('prefix',BL.mda.prefix())
    kwargs.setdefault('plotType',['slit1','wire','flux','monoVslit'])
    kwargs.setdefault('ref_firstScanNum',None)
    kwargs.setdefault('ref_fpath',None)
    kwargs.setdefault('ref_branch',branch)
    kwargs.setdefault('ref_filepath',BL.mda.filepath())
    kwargs.setdefault('ref_prefix',BL.mda.prefix())
    kwargs.setdefault('debug',False)

    detCA4,detH,detV,detDiode=plot_StartOfTheWeek_DetDict(branch)
    ref_detCA4,ref_detH,ref_detV,ref_detDiode=plot_StartOfTheWeek_DetDict(kwargs['ref_branch']) 
    
    figure, axes = plt.subplots(nrows=1, ncols=2,figsize=(10,3))
    for i,t in enumerate(['H-wire','V-wire']):
        d=IEXdata(scanNum,path=kwargs['filepath'],prefix=kwargs['prefix'],q=1)
        #niceplot(d.mda[scanNum].det[DetDict[branch][1+i]],marker='x',label=str(scanNum))
        detNum=plot_StartOfTheWeek_DetDict(branch)[1+i]
        axes[i].plot(d.mda[scanNum].det[detNum].scale['x'], d.mda[scanNum].det[detNum].data,marker='x',label=str(scanNum))
        scanNum+=1
        if kwargs['ref_firstScanNum'] is not None:
            ref_scanNum=kwargs['ref_firstScanNum']+i
            ref_d=IEXdata(ref_scanNum,path=kwargs['ref_filepath'],prefix=kwargs['ref_prefix'],q=1)
            #niceplot(ref_d.mda[ref_scanNum].det[DetDict[kwargs["ref_branch"]][1+i]],marker='x',label=str(ref_scanNum))
            detNum=plot_StartOfTheWeek_DetDict(kwargs["ref_branch"])[1+i]
            axes[i].plot(ref_d.mda[ref_scanNum].det[detNum].scale['x'], ref_d.mda[ref_scanNum].det[detNum].data,marker='x',label=str(ref_scanNum))
        axes[i].grid(color='lightgray', linestyle='-', linewidth=0.5)
        axes[i].title.set_text(t)
        axes[i].legend()
    plt.show()    
        
def plot_StartOfTheWeek_flux(branch,scanNum,**kwargs):
    """
    plots the wire scans
    scanNum = wire-H scanNum
    """   
    kwargs.setdefault('filepath',BL.mda.filepath())
    kwargs.setdefault('prefix',BL.mda.prefix())
    kwargs.setdefault('plotType',['slit1','wire','flux','monoVslit'])
    kwargs.setdefault('ref_firstScanNum',None)
    kwargs.setdefault('ref_fpath',None)
    kwargs.setdefault('ref_branch',branch)
    kwargs.setdefault('ref_filepath',BL.mda.filepath())
    kwargs.setdefault('ref_prefix',BL.mda.prefix())
    kwargs.setdefault('debug',False) 

    detCA4,detH,detV,detDiode=plot_StartOfTheWeek_DetDict(branch)
    ref_detCA4,ref_detH,ref_detV,ref_detDiode=plot_StartOfTheWeek_DetDict(kwargs['ref_branch'])     
    
    for t in ['ID peak @ 500eV']:
        d=IEXdata(scanNum,path=kwargs['filepath'],prefix=kwargs['prefix'],q=1)
        niceplot(d.mda[scanNum].det[detDiode],marker='x',label=str(scanNum))
        scanNum+=1
        if kwargs['ref_firstScanNum'] is not None:
            ref_scanNum=kwargs['ref_firstScanNum']+2
            ref_d=IEXdata(ref_scanNum,path=kwargs['ref_filepath'],prefix=kwargs['ref_prefix'],q=1)
            niceplot(ref_d.mda[ref_scanNum].det[ref_detDiode],marker='x',label=str(ref_scanNum))
            ref_scanNum+=1
        plt.grid(color='lightgray', linestyle='-', linewidth=0.5)
        plt.title(t)
        plt.legend()
        plt.show()
    
def plot_MonoVsSlit(branch,ScanNum,detDiode,pnt_first,pnt_last,norm=True,filepath=None,prefix=None,scanIOC=None):
    """
    Plots Scan_MonoVsSlit to determine the steering from M0/M1 
        To plot the full range (pnt_first=0, pnt_last=inf)
    plot_StartofWeek_mono(branch,FirstScanNum+4,pnt_first,pnt_last)
    
    filepath: by default plot scans for the current data folder (as defined in BL_ioc() ScanRecord SaveData)
    or specified folder path ending with '/':
        e.g. user : filepath='/net/s29data/export/data_29idc/2018_2/UserName/mda/'
        e.g. staff: filepath='/net/s29data/export/data_29idb/2018_2/mda/'
    prefix: by default, uses prefix as defined in ScanRecord ("mda_")
    """
    x,y,z,x_name,y_name,z_name=mda_2D(ScanNum,detDiode,filepath,prefix,scanIOC)
    Which=str(y_name)[10:12]
    if pnt_last is inf:
        pnt_last=len(z)-1
    for i in range(pnt_first,pnt_last+1):
        maxvalue=max(z[i])
        if norm == True:
            plt.plot(x,z[i]/maxvalue,label='#'+str(i)+': '+str(round(y[i],2)))
        else:
            plt.plot(x,z[i],label='#'+str(i)+': '+str(round(y[i],2)))
        plt.legend(bbox_to_anchor=(1, 0), loc='lower left', ncol=2,shadow=True, title="ScanNum: "+str(ScanNum)+"\nSlit-"+Which+" position", fancybox=True)
        #plt.legend(loc='lower left',ncol=2, shadow=True, title="ScanNum: "+str(ScanNum)+"\nSlit-"+Which+" position", fancybox=True) 
        plt.grid(color='lightgray', linestyle='-', linewidth=0.5)
    plt.show()
   

def read_flux(FileName='Flux_Curves.txt',FilePath="/home/beams22/29IDUSER/Documents/User_Macros/Macros_29id/IEX_Dictionaries/"):
    print('test')
    with open(join(FilePath, FileName)) as f:
        for c,line in enumerate(f.readlines()):
            if line[0] == '=':
                lastdate=line[8:16]
                print(lastdate)
            if line[0] == '-' and line[-2] == 'x':
                axis='x'
                print(axis)
            line_x=line
            if line[0] == '-' and line[-2] == 'y':
                axis='y'
                print(axis)
            line_y=line
        mydict_x=ast.literal_eval(line_x)
        mydict_y=ast.literal_eval(line_y)
    return mydict_x,mydict_y


